@@ -81,12 +81,14 @@ static void locus_test_data(GtQueue *queue);
 /**
  * @function Track order function for PNG graphics.
  */
+#ifndef WITHOUT_CAIRO
 static int locus_track_order(const char *s1, const char *s2, void *data)
 {
   if(strstr(s1, "Reference") == NULL)
     return 1;
   return -1;
 }
+#endif
 
 /**
  * @function For a set of features, we can construct a graph where each node
@@ -462,7 +464,7 @@ void agn_locus_filter_parse(FILE *filterfile, GtArray *filters)
       if(strcmp(srcstr, "R") == 0)
         filter.src = REFERENCESOURCE;
       else if(strcmp(srcstr, "P") == 0)
-        filter.src = REFERENCESOURCE;
+        filter.src = PREDICTIONSOURCE;
       else
       {
         fprintf(stderr, "[AgnLocus::agn_locus_filter_parse] invalid source "
@@ -471,7 +473,8 @@ void agn_locus_filter_parse(FILE *filterfile, GtArray *filters)
       }
     }
 
-    gt_array_add(filters, filter);
+    if(filter.operator != AGN_LOCUS_FILTER_NO)
+      gt_array_add(filters, filter);
   }
 }
 
@@ -483,33 +486,59 @@ bool agn_locus_filter_test(AgnLocus *locus, AgnLocusFilter *filter)
     return true;
 
   value = filter->function(locus, filter->src);
-  switch(filter->operator)
+  if(filter->operator == AGN_LOCUS_FILTER_EQ)
   {
-    case AGN_LOCUS_FILTER_EQ:
-      if(value == filter->testvalue) return true;
-      break;
-    case AGN_LOCUS_FILTER_NE:
-      if(value != filter->testvalue) return true;
-      break;
-    case AGN_LOCUS_FILTER_GT:
-      if(value > filter->testvalue) return true;
-      break;
-    case AGN_LOCUS_FILTER_GE:
-      if(value >= filter->testvalue) return true;
-      break;
-    case AGN_LOCUS_FILTER_LT:
-      if(value < filter->testvalue) return true;
-      break;
-    case AGN_LOCUS_FILTER_LE:
-      if(value <= filter->testvalue) return true;
-      break;
-    case AGN_LOCUS_FILTER_NO: // Added this entry to case statement because
-      return true;            // of compiler complaint
-      break;
+    if(value == filter->testvalue)
+      return true;
+  }
+  else if(filter->operator == AGN_LOCUS_FILTER_NE)
+  {
+    if(value != filter->testvalue)
+      return true;
+  }
+  else if(filter->operator == AGN_LOCUS_FILTER_GT)
+  {
+    if(value > filter->testvalue)
+      return true;
+  }
+  else if(filter->operator == AGN_LOCUS_FILTER_GE)
+  {
+    if(value >= filter->testvalue)
+      return true;
+  }
+  else if(filter->operator == AGN_LOCUS_FILTER_LT)
+  {
+    if(value < filter->testvalue)
+      return true;
+  }
+  else if(filter->operator == AGN_LOCUS_FILTER_LE)
+  {
+    if(value <= filter->testvalue)
+      return true;
+  }
+  else if(filter->operator == AGN_LOCUS_FILTER_NO)
+  {
+    return true;
   }
   return false;
 }
 
+GtArray *agn_locus_get(AgnLocus *locus)
+{
+  GtFeatureNode *locusfn = gt_feature_node_cast(locus);
+  GtFeatureNodeIterator *iter = gt_feature_node_iterator_new_direct(locusfn);
+  GtFeatureNode *child;
+  GtArray *children = gt_array_new( sizeof(GtFeatureNode *) );
+  for(child  = gt_feature_node_iterator_next(iter);
+      child != NULL;
+      child  = gt_feature_node_iterator_next(iter))
+  {
+    gt_array_add(children, child);
+  }
+  gt_feature_node_iterator_delete(iter);
+  return children;
+}
+
 GtArray *agn_locus_get_unique_pred_cliques(AgnLocus *locus)
 {
   return gt_genome_node_get_user_data(locus, "uniqpred");
@@ -583,6 +612,40 @@ GtUword agn_locus_gene_num(AgnLocus *locus, AgnComparisonSource src)
   return count;
 }
 
+int agn_locus_inner_orientation(AgnLocus *left, AgnLocus *right)
+{
+  agn_assert(left && right);\
+
+  GtArray *leftgenes = agn_locus_get(left);
+  GtArray *rightgenes = agn_locus_get(right);
+  agn_assert(gt_array_size(leftgenes) > 0);
+  agn_assert(gt_array_size(rightgenes) > 0);
+
+  GtFeatureNode *leftgene = NULL;
+  GtUword end = 0;
+  while(gt_array_size(leftgenes) > 0)
+  {
+    GtGenomeNode **testgene = gt_array_pop(leftgenes);
+    if(leftgene == NULL || gt_genome_node_get_end(*testgene) > end)
+      leftgene = gt_feature_node_cast(*testgene);
+  }
+  gt_array_delete(leftgenes);
+  GtFeatureNode *rightgene = *(GtFeatureNode **)gt_array_get(rightgenes, 0);
+  gt_array_delete(rightgenes);
+
+  GtStrand leftstrand = gt_feature_node_get_strand(leftgene);
+  GtStrand rightstrand = gt_feature_node_get_strand(rightgene);
+
+  if(leftstrand == GT_STRAND_FORWARD && rightstrand == GT_STRAND_FORWARD)
+    return 0;
+  else if(leftstrand == GT_STRAND_FORWARD && rightstrand == GT_STRAND_REVERSE)
+    return 1;
+  else if(leftstrand == GT_STRAND_REVERSE && rightstrand == GT_STRAND_FORWARD)
+    return 2;
+  else
+    return 3;
+}
+
 GtArray *agn_locus_mrnas(AgnLocus *locus, AgnComparisonSource src)
 {
   GtArray *mrnas = gt_array_new( sizeof(GtFeatureNode *) );
@@ -860,6 +923,12 @@ void agn_locus_print_transcript_mapping(AgnLocus *locus, FILE *outstream)
 
 void agn_locus_set_range(AgnLocus *locus, GtUword start, GtUword end)
 {
+  if(start > end)
+  {
+    GtStr *seqid = gt_genome_node_get_seqid(locus);
+    fprintf(stderr, "%s %lu %lu\n", gt_str_get(seqid), start, end);
+  }
+  agn_assert(start <= end);
   GtRange range = { start, end };
   gt_genome_node_set_range(locus, &range);
 }
