@@ -1,6 +1,6 @@
 /**
 
-Copyright (c) 2010-2014, Daniel S. Standage and CONTRIBUTORS
+Copyright (c) 2010-2015, Daniel S. Standage and CONTRIBUTORS
 
 The AEGeAn Toolkit is distributed under the ISC License. See
 the 'LICENSE' file in the AEGeAn source code distribution or
@@ -30,16 +30,17 @@ struct AgnLocusStream
   GtNodeStream *in_stream;
   GtUword delta;
   GtUword count;
-  bool skip_empty;
+  bool skip_iiLoci;
   int endmode;
   GtFeatureIndex *seqranges;
   AgnLocus *prev_locus;
   GtQueue *locusqueue;
   GtGenomeNode *buffer;
   GtStr *source;
-  GtStr *idformat;
+  GtStr *nameformat;
   char *refrfile;
   char *predfile;
+  FILE *ilenfile;
 };
 
 //------------------------------------------------------------------------------
@@ -139,16 +140,17 @@ GtNodeStream *agn_locus_stream_new(GtNodeStream *in_stream, GtUword delta)
   stream->in_stream = gt_node_stream_ref(in_stream);
   stream->delta = delta;
   stream->count = 0;
-  stream->skip_empty = false;
+  stream->skip_iiLoci = false;
   stream->endmode = 0;
   stream->seqranges = gt_feature_index_memory_new();
   stream->prev_locus = NULL;
   stream->locusqueue = gt_queue_new();
   stream->buffer = NULL;
   stream->source = gt_str_new_cstr("AEGeAn::AgnLocusStream");
-  stream->idformat = gt_str_new_cstr("locus%lu");
+  stream->nameformat = NULL;
   stream->refrfile = NULL;
   stream->predfile = NULL;
+  stream->ilenfile = NULL;
   return ns;
 }
 
@@ -158,17 +160,18 @@ void agn_locus_stream_set_endmode(AgnLocusStream *stream, int endmode)
   stream->endmode = endmode;
 }
 
-void agn_locus_stream_set_idformat(AgnLocusStream *stream, const char *format)
+void agn_locus_stream_set_name_format(AgnLocusStream *stream, const char *fmt)
 {
-  agn_assert(stream && format);
-  gt_str_delete(stream->idformat);
-  stream->idformat = gt_str_new_cstr(format);
+  agn_assert(stream && fmt);
+  if(stream->nameformat)
+    gt_str_delete(stream->nameformat);
+  stream->nameformat = gt_str_new_cstr(fmt);
 }
 
-void agn_locus_stream_skip_empty_loci(AgnLocusStream *stream)
+void agn_locus_stream_skip_iiLoci(AgnLocusStream *stream)
 {
   agn_assert(stream);
-  stream->skip_empty = true;
+  stream->skip_iiLoci = true;
 }
 
 void agn_locus_stream_set_source(AgnLocusStream *stream, const char *source)
@@ -185,6 +188,11 @@ bool agn_locus_stream_unit_test(AgnUnitTest *test)
   return agn_unit_test_success(test);
 }
 
+void agn_locus_stream_track_ilens(AgnLocusStream *stream, FILE *ilenfile)
+{
+  stream->ilenfile = ilenfile;
+}
+
 static int locus_stream_add_feature(AgnLocusStream *stream, AgnLocus *locus,
                                     GtFeatureNode *feature, GtError *error)
 {
@@ -241,7 +249,7 @@ static void locus_stream_extend(AgnLocusStream *stream, AgnLocus *locus)
   GtRange seqrange;
   gt_feature_index_get_range_for_seqid(stream->seqranges, &seqrange,
                                        gt_str_get(seqid), NULL);
-  GtStr *prev_seqid;
+  GtStr *prev_seqid = NULL;
   if(stream->prev_locus)
     prev_seqid = gt_genome_node_get_seqid(stream->prev_locus);
 
@@ -252,14 +260,15 @@ static void locus_stream_extend(AgnLocusStream *stream, AgnLocus *locus)
     {
       agn_locus_set_range(locus, locusrange.start - stream->delta,
                           locusrange.end);
-      if(stream->endmode >= 0 && !stream->skip_empty)
+      if(stream->endmode >= 0 && !stream->skip_iiLoci)
       {
-        AgnLocus *emptylocus = agn_locus_new(seqid);
-        agn_locus_set_range(emptylocus, seqrange.start,
-                            locusrange.start - stream->delta - 1);
-        GtFeatureNode *elfn = (GtFeatureNode *)emptylocus;
-        gt_feature_node_add_attribute(elfn, "fragment", "true");
-        gt_queue_add(stream->locusqueue, emptylocus);
+        AgnLocus *iilocus = agn_locus_new(seqid);
+        GtRange irange = { seqrange.start,
+                           locusrange.start - stream->delta - 1 };
+        agn_locus_set_range(iilocus, irange.start, irange.end);
+        GtFeatureNode *iilocfn = gt_feature_node_cast(iilocus);
+        gt_feature_node_add_attribute(iilocfn, "fragment", "true");
+        gt_queue_add(stream->locusqueue, iilocus);
       }
     }
     else
@@ -271,40 +280,102 @@ static void locus_stream_extend(AgnLocusStream *stream, AgnLocus *locus)
   // Handle internal loci
   if(stream->prev_locus && gt_str_cmp(seqid, prev_seqid) == 0)
   {
+    GtFeatureNode *prevfn = gt_feature_node_cast(stream->prev_locus);
+    GtFeatureNode *locusfn = gt_feature_node_cast(locus);
     GtRange prev_range = gt_genome_node_get_range(stream->prev_locus);
-    if(prev_range.end + stream->delta >= locusrange.start)
+    bool ovlp1 = prev_range.end + stream->delta >= locusrange.start;
+    bool ovlp2 = prev_range.end >= locusrange.start;
+    if(ovlp1 || ovlp2)
     {
-      agn_locus_set_range(stream->prev_locus, prev_range.start,
-                          locusrange.start - 1);
-      agn_locus_set_range(locus, prev_range.end + 1, locusrange.end);
+      GtUword newend = prev_range.end + stream->delta;
+      if(newend > seqrange.end)
+        newend = seqrange.end;
+      agn_locus_set_range(stream->prev_locus, prev_range.start, newend);
+
+      GtUword newstart = locusrange.start - stream->delta;
+      if(seqrange.start + stream->delta > locusrange.start)
+        newstart = seqrange.start;
+      agn_locus_set_range(locus, newstart, locusrange.end);
+
+      GtUword overlap = newend - newstart + 1;
+      char ovrlp[16];
+      sprintf(ovrlp, "%lu", overlap);
+      gt_feature_node_add_attribute(prevfn, "right_overlap", ovrlp);
+      gt_feature_node_add_attribute(locusfn, "left_overlap", ovrlp);
+      gt_feature_node_add_attribute(prevfn, "iiLocus_exception",
+                                    "delta-overlap-gene");
+
+      if(stream->ilenfile != NULL)
+        fprintf(stream->ilenfile, "%s\t0\n", gt_str_get(seqid));
+      gt_feature_node_add_attribute(prevfn, "riil", "0");
+      gt_feature_node_add_attribute(locusfn, "liil", "0");
     }
     else if(prev_range.end + (2*stream->delta) >= locusrange.start)
     {
-      agn_locus_set_range(stream->prev_locus, prev_range.start,
-                          prev_range.end + stream->delta);
+      GtUword newend = prev_range.end + stream->delta;
+      if(newend > seqrange.end)
+        newend = seqrange.end;
+      agn_locus_set_range(stream->prev_locus, prev_range.start, newend);
       agn_locus_set_range(locus, locusrange.start - stream->delta,
                           locusrange.end);
+
+      GtUword overlap = prev_range.end-locusrange.start+(2*stream->delta)+1;
+      char ovrlp[16];
+      sprintf(ovrlp, "%lu", overlap);
+      gt_feature_node_add_attribute(prevfn, "right_overlap", ovrlp);
+      gt_feature_node_add_attribute(locusfn, "left_overlap", ovrlp);
+      gt_feature_node_add_attribute(prevfn, "iiLocus_exception",
+                                    "delta-overlap-delta");
+
+      if(stream->ilenfile != NULL)
+        fprintf(stream->ilenfile, "%s\t0\n", gt_str_get(seqid));
+      gt_feature_node_add_attribute(prevfn, "riil", "0");
+      gt_feature_node_add_attribute(locusfn, "liil", "0");
     }
     else if(prev_range.end + (3*stream->delta) >= locusrange.start)
     {
       GtUword midpoint = (prev_range.end + locusrange.start) / 2;
       agn_locus_set_range(stream->prev_locus, prev_range.start, midpoint);
       agn_locus_set_range(locus, midpoint + 1, locusrange.end);
+      gt_feature_node_add_attribute(prevfn, "iiLocus_exception",
+                                    "delta-re-extend");
+
+      if(stream->ilenfile != NULL)
+        fprintf(stream->ilenfile, "%s\t0\n", gt_str_get(seqid));
+      gt_feature_node_add_attribute(prevfn, "riil", "0");
+      gt_feature_node_add_attribute(locusfn, "liil", "0");
     }
     else
     {
-      agn_locus_set_range(stream->prev_locus, prev_range.start,
-                          prev_range.end + stream->delta);
+      GtUword newend = prev_range.end + stream->delta;
+      if(newend > seqrange.end)
+        newend = seqrange.end;
+      agn_locus_set_range(stream->prev_locus, prev_range.start, newend);
       agn_locus_set_range(locus, locusrange.start - stream->delta,
                           locusrange.end);
 
-      if(stream->endmode <= 0 && !stream->skip_empty)
+      if(stream->endmode <= 0 && !stream->skip_iiLoci)
       {
-        AgnLocus *emptylocus = agn_locus_new(seqid);
-        agn_locus_set_range(emptylocus,
-                            prev_range.end + stream->delta + 1,
-                            locusrange.start - stream->delta - 1);
-        gt_queue_add(stream->locusqueue, emptylocus);
+        AgnLocus *iilocus = agn_locus_new(seqid);
+        GtRange irange = { prev_range.end + stream->delta + 1,
+                           locusrange.start - stream->delta - 1 };
+        agn_locus_set_range(iilocus, irange.start, irange.end);
+
+        if(stream->ilenfile != NULL)
+        {
+          fprintf(stream->ilenfile, "%s\t%lu\n", gt_str_get(seqid),
+                  gt_range_length(&irange));
+        }
+        char iilocuslen[32];
+        sprintf(iilocuslen, "%lu", gt_range_length(&irange));
+        gt_feature_node_add_attribute(prevfn, "riil", iilocuslen);
+        gt_feature_node_add_attribute(locusfn, "liil", iilocuslen);
+
+        const char *orientstrs[] = { "FF", "FR", "RF", "RR" };
+        int orient = agn_locus_inner_orientation(stream->prev_locus, locus);
+        GtFeatureNode *iilocfn = gt_feature_node_cast(iilocus);
+        gt_feature_node_set_attribute(iilocfn, "fg_orient", orientstrs[orient]);
+        gt_queue_add(stream->locusqueue, iilocus);
       }
     }
   }
@@ -324,7 +395,7 @@ static void locus_stream_extend(AgnLocusStream *stream, AgnLocus *locus)
     {
       agn_locus_set_range(locus, locusrange.start,
                           locusrange.end + stream->delta);
-      if(stream->endmode >= 0 && !stream->skip_empty)
+      if(stream->endmode >= 0 && !stream->skip_iiLoci)
       {
         AgnLocus *term_locus = agn_locus_new(seqid);
         agn_locus_set_range(term_locus,
@@ -371,17 +442,12 @@ static int locus_stream_fn_handler(AgnLocusStream *stream, GtGenomeNode **gn,
       break;
     }
 
-    GtStr *newseqid = gt_genome_node_get_seqid(*gn);
-    GtRange newrange = gt_genome_node_get_range(*gn);
     GtUword i;
     bool overlap = false;
     for(i = 0; i < gt_array_size(current_locus); i++)
     {
       GtGenomeNode **oldfeature = gt_array_get(current_locus, i);
-      GtStr *oldseqid = gt_genome_node_get_seqid(*oldfeature);
-      GtRange oldrange = gt_genome_node_get_range(*oldfeature);
-      if(gt_str_cmp(newseqid, oldseqid) == 0 &&
-         gt_range_overlap(&newrange, &oldrange))
+      if(agn_overlap_ilocus(*gn, *oldfeature, 1, false))
       {
         overlap = true;
         break;
@@ -441,7 +507,8 @@ static void locus_stream_free(GtNodeStream *ns)
   gt_feature_index_delete(stream->seqranges);
   gt_queue_delete(stream->locusqueue);
   gt_str_delete(stream->source);
-  gt_str_delete(stream->idformat);
+  if(stream->nameformat)
+    gt_str_delete(stream->nameformat);
   gt_free(stream->refrfile);
   gt_free(stream->predfile);
 }
@@ -451,10 +518,13 @@ static void locus_stream_mint(AgnLocusStream *stream, AgnLocus *locus)
   agn_assert(stream);
   stream->count++;
 
-  char locusid[256];
-  sprintf(locusid, gt_str_get(stream->idformat), stream->count);
-  gt_feature_node_set_attribute((GtFeatureNode *)locus, "ID", locusid);
   gt_feature_node_set_source((GtFeatureNode *)locus, stream->source);
+  if(stream->nameformat)
+  {
+    char locusname[256];
+    sprintf(locusname, gt_str_get(stream->nameformat), stream->count);
+    gt_feature_node_set_attribute((GtFeatureNode *)locus, "Name", locusname);
+  }
 
   GtArray *types = gt_array_new( sizeof(const char *) );
   GtHashmap *countsbytype = gt_hashmap_new(GT_HASH_STRING, gt_free_func,
@@ -506,9 +576,11 @@ static void locus_stream_mint(AgnLocusStream *stream, AgnLocus *locus)
   {
     const char **attrkey = gt_array_get(types, i);
     GtUword *attrvalue = gt_hashmap_get(countsbytype, *attrkey);
+    char key[128];
     char value[32];
+    sprintf(key, "child_%s", *attrkey);
     sprintf(value, "%lu", *attrvalue);
-    gt_feature_node_set_attribute(locusfn, *attrkey, value);
+    gt_feature_node_set_attribute(locusfn, key, value);
   }
 
   gt_hashmap_delete(countsbytype);
